`timescale 1ns / 1ps
`include "parameters.v"

module test_io_expander_v;

	// Inputs
	reg clk;
	reg reset;
	reg SCLK;
	reg SI;
	reg SS;

	// Outputs
	wire SO;
	wire INT;

	// Bidirs
	wire [15:0] gpio_pins;

	reg [`GPIO_PINS-1:0]gpio_pins_going_in;
	reg [`GPIO_PINS-1:0]gpio_pins_going_in_temp;
	wire [`GPIO_PINS-1:0]gpio_pins_coming_out;
	reg [`GPIO_PINS-1:0]direction_register;

	reg [3:0]test;
	reg [15:0]keypress;
	wire [3:0]keypress_pins_out;

	// Instantiate the Unit Under Test (UUT)
	io_expander uut (
		.clk(clk), 
		.reset(reset), 
		.SCLK(SCLK), 
		.SI(SI), 
		.SO(SO), 
		.SS(SS), 
		.INT(INT), 
		.gpio_pins(gpio_pins)
	);

// mask for the output pins
assign gpio_pins[0] = (direction_register [0]) ? (gpio_pins_going_in[0] | keypress_pins_out[0]) : 1'bz;
assign gpio_pins[1] = (direction_register [1]) ? (gpio_pins_going_in[1] | keypress_pins_out[1]) : 1'bz;
assign gpio_pins[2] = (direction_register [2]) ? (gpio_pins_going_in[2] | keypress_pins_out[2]) : 1'bz;
assign gpio_pins[3] = (direction_register [3]) ? (gpio_pins_going_in[3] | keypress_pins_out[3]) : 1'bz;
assign gpio_pins[4] = (direction_register [4]) ? gpio_pins_going_in[4] : 1'bz;
assign gpio_pins[5] = (direction_register [5]) ? gpio_pins_going_in[5] : 1'bz;
assign gpio_pins[6] = (direction_register [6]) ? gpio_pins_going_in[6] : 1'bz;
assign gpio_pins[7] = (direction_register [7]) ? gpio_pins_going_in[7] : 1'bz;
assign gpio_pins[8] = (direction_register [8]) ? gpio_pins_going_in[8] : 1'bz;
assign gpio_pins[9] = (direction_register [9]) ? gpio_pins_going_in[9] : 1'bz;
assign gpio_pins[10] = (direction_register [10]) ? gpio_pins_going_in[10] : 1'bz;
assign gpio_pins[11] = (direction_register [11]) ? gpio_pins_going_in[11] : 1'bz;
assign gpio_pins[12] = (direction_register [12]) ? gpio_pins_going_in[12] : 1'bz;
assign gpio_pins[13] = (direction_register [13]) ? gpio_pins_going_in[13] : 1'bz;
assign gpio_pins[14] = (direction_register [14]) ? gpio_pins_going_in[14] : 1'bz;
assign gpio_pins[15] = (direction_register [15]) ? gpio_pins_going_in[15] : 1'bz;

// mask for the input pins
assign gpio_pins_coming_out[0] = (!direction_register [0]) ? gpio_pins[0] : 1'bz;
assign gpio_pins_coming_out[1] = (!direction_register [1]) ? gpio_pins[1] : 1'bz;
assign gpio_pins_coming_out[2] = (!direction_register [2]) ? gpio_pins[2] : 1'bz;
assign gpio_pins_coming_out[3] = (!direction_register [3]) ? gpio_pins[3] : 1'bz;
assign gpio_pins_coming_out[4] = (!direction_register [4]) ? gpio_pins[4] : 1'bz;
assign gpio_pins_coming_out[5] = (!direction_register [5]) ? gpio_pins[5] : 1'bz;
assign gpio_pins_coming_out[6] = (!direction_register [6]) ? gpio_pins[6] : 1'bz;
assign gpio_pins_coming_out[7] = (!direction_register [7]) ? gpio_pins[7] : 1'bz;
assign gpio_pins_coming_out[8] = (!direction_register [8]) ? gpio_pins[8] : 1'bz;
assign gpio_pins_coming_out[9] = (!direction_register [9]) ? gpio_pins[9] : 1'bz;
assign gpio_pins_coming_out[10] = (!direction_register [10]) ? gpio_pins[10] : 1'bz;
assign gpio_pins_coming_out[11] = (!direction_register [11]) ? gpio_pins[11] : 1'bz;
assign gpio_pins_coming_out[12] = (!direction_register [12]) ? gpio_pins[12] : 1'bz;
assign gpio_pins_coming_out[13] = (!direction_register [13]) ? gpio_pins[13] : 1'bz;
assign gpio_pins_coming_out[14] = (!direction_register [14]) ? gpio_pins[14] : 1'bz;
assign gpio_pins_coming_out[15] = (!direction_register [15]) ? gpio_pins[15] : 1'bz;

/* converts keypresses to row values depending on column inputs */
keygrid_4x4 keypresser(
	.presses(keypress), 
	.column_ins(gpio_pins_coming_out[7:4]),
	.row_outs(keypress_pins_out));

initial begin
		// Initialize Inputs
		clk = 0;
		reset = 1;
		SCLK = 0;
		SI = 0;
		SS = 1;

		// Wait 100 ns for global reset to finish
		#100;
        
		// Add stimulus here

		reset = 0;
		#10;

		test = 2;
		
		if (test == 1)
		begin
			//----------------------------------------------------
			// Description sends 0000000011111111 to say address 1 - write - means setting first half to interrupt
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 11111111 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 1; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 1; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 1; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 1; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 1; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 1; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 1; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 00000000 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			
			direction_register = 16'b0000111111111111;
			//----------------------------------------------------
			// Description sends 0000111111111111 to say address 2 - write - means setting first byte and half second byte to read direction
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 1; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 11111111 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 1; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 1; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 1; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 1; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 1; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 1; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 1; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 00001111 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 1; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 1; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 1; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			
			//----------------------------------------------------
			// Description sends 1010000000000000 to say address 3 - write - means of the 4 write bits set them to 1010
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 1; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 00000000 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 10100000 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 1; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 1; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
						
			//----------------------------------------------------
			// Description sends 0000000100000000 to say address 0 - write - no keypad, and to start working
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 00000000 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 00000001 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 1; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			
			#500;
			//	        1234567812345678	
			gpio_pins_going_in = 16'bzzzz000000000000;
			#50;
			gpio_pins_going_in = 16'bZZZZ000100000000;
			#50;
			gpio_pins_going_in = 16'bZZZZ000100010000;
			#50;
			gpio_pins_going_in = 16'bZZZZ000100010001;
			#50;
	
			#500;
			
			//----------------------------------------------------
			// Description sends 00000111 to say address 3 -> read
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 1; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 1; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit read packet
			//----------------------------------------------------
			//----------------------------------------------------
			// READING
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// 1bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 9bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 10
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 11
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 12
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 13
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 14
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 15
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 16
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 16 bit packet
			//----------------------------------------------------
		end
		else if (test == 2)
		begin
			gpio_pins_going_in = 16'b0000zzzzzzzz0000;
			keypress = 16'b0000000000000000;
			//----------------------------------------------------
			// Description sends 1010000000000000 to say address 1 - write - means setting two ports to interrupt
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 00000000 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 10100000 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 1; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 1; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			
			direction_register = 16'b1111000000001111;
			//----------------------------------------------------
			// Description sends 1111000000000000 to say address 2 - write - means setting first half second byte to read direction
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 1; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 00000000 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 11110000 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 1; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 1; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 1; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 1; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			
			//----------------------------------------------------
			// Description sends 0000101000000000 to say address 3 - write - means of the 4 write bits set them to 1010
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 1; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 00000000 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 10100000 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 1; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
						
			//----------------------------------------------------
			// Description sends 0000000111000100 to say address 0 - write - no keypad, and to start working 4x4 keypad
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 11000100 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 0; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 1; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 1; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 1; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			//----------------------------------------------------
			// Description sends 00000001 as data
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 1; // lsb data bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 0; // 2nd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 3rd data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 0; // 4th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 5th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 6th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 7th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 8th data bit 
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit packet
			//----------------------------------------------------
			
			#500;
			gpio_pins_going_in = 16'b0000zzzzzzzz0000;
			keypress = 16'b0000000000000000;

			#200;
			//	                 1234567890123456	
			keypress = 16'b0000000000000001;
			#200;
	
			keypress = 16'b0001000000100000;
			#600;

			keypress = 16'b0010000000100100;
			#500;
			
			//----------------------------------------------------
			// Description sends 00001011 to say address 5 -> read fifo
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 1; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 1; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit read packet
			//----------------------------------------------------
			//----------------------------------------------------
			// READING
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// 1bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 9bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 10
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 11
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 12
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 13
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 14
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 15
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 16
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 16 bit packet
			//----------------------------------------------------
			
			//----------------------------------------------------
			// Description sends 00001011 to say address 5 -> read fifo
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 1; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 1; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit read packet
			//----------------------------------------------------
			//----------------------------------------------------
			// READING
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// 1bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 9bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 10
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 11
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 12
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 13
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 14
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 15
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 16
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 16 bit packet
			//----------------------------------------------------

			//----------------------------------------------------
			// Description sends 00001011 to say address 5 -> read fifo
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// send a byte
			// 1bit
			SI = 1; // first bit 1 = Read 0 = Write
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SI = 1; // lsb bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SI = 0; // 2nd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SI = 1; // 3rd bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SI = 0; // 4th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SI = 0; // 5th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SI = 0; // 6th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SI = 0; // 7th bit of address
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 8 bit read packet
			//----------------------------------------------------
			//----------------------------------------------------
			// READING
			//----------------------------------------------------
			SS = 0; // active low selects the device
			#5;
			// 1bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 2bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 3bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 4bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 5bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 6bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 7bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 8bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 9bit
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 10
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 11
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 12
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 13
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 14
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 15
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			// 16
			SCLK = 1;
			#25;
			SCLK = 0;
			#25;
			//----------------------------------------------------
			// End 16 bit packet
			//----------------------------------------------------
		end
	end
      
	always
	begin
		#5 clk = ~clk;
	end
endmodule
